// -*- C++ -*-
/*!
 * @file  HumanDetection_LRF.cpp 
* @brief HumanDetection_LRF compornent
 * $Date$ 
 * @author Nemoto,Mizoguchi lab,Tokyo University of Science
 * @mail:j7507640@ed.noda.tus.ac.jp
 * $Id$ 
 */
#include "HumanDetection_LRF.h"

// Module specification
// <rtc-template block="module_spec">
static const char* humandetection_lrf_spec[] =
  {
    "implementation_id", "HumanDetection_LRF",
    "type_name",         "HumanDetecLRF",
    "description",       "HumanDetection_LRF compornent",
    "version",           "1.0",
    "vendor",            "Nemoto,Mizoguchi Lab,Tokyo University of Science",
    "category",          "RobotModule",
    "activity_type",     "SPORADIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "10",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    ""
  };
// </rtc-template>

HumanDetection_LRF::HumanDetection_LRF(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_UrgDataIn("UrgData", m_UrgData),
    m_HumanPosXYOut("HumanPosXY", m_HumanPosXY),
    m_HumanPosOut("HumanPos", m_HumanPos),
    m_ObstacleOut("Obstacle", m_Obstacle),
    m_HumanObjectOut("HumanObject", m_HumanObject),
    m_ObstacleObjectOut("ObstacleObject", m_ObstacleObject),
    m_TrackingPortPort("TrackingPort"),
    m_LRFInfoPort("LRFInfo"),
    m_VisionPosIn("VisionPos", m_VisionPos),
    m_TargetOdometryIn("TargetOdometry", m_TargetOdometry)
    // </rtc-template>
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("UrgData", m_UrgDataIn);
  registerInPort("VisionPos", m_VisionPosIn);
  registerInPort("TargetOdometry", m_TargetOdometryIn);
  // Set OutPort buffer
  registerOutPort("HumanPosXY", m_HumanPosXYOut);
  registerOutPort("HumanPos", m_HumanPosOut);
  registerOutPort("Obstacle", m_ObstacleOut);
  registerOutPort("HumanObject", m_HumanObjectOut);
  registerOutPort("ObstacleObject", m_ObstacleObjectOut);

  // Set service provider to Ports
  m_TrackingPortPort.registerProvider("LRFTracking", "LRFTracking", m_LRFTracking);

  // Set service consumers to Ports
  m_LRFInfoPort.registerConsumer("LRF", "LRFInfo", m_LRF);

  // Set CORBA Service Ports
  registerPort(m_TrackingPortPort);
  registerPort(m_LRFInfoPort);

  // </rtc-template>

}

HumanDetection_LRF::~HumanDetection_LRF()
{
}


RTC::ReturnCode_t HumanDetection_LRF::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable

	  // </rtc-template>
  return RTC::RTC_OK;
}



RTC::ReturnCode_t HumanDetection_LRF::onFinalize()
{
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t HumanDetection_LRF::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t HumanDetection_LRF::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

RTC::ReturnCode_t HumanDetection_LRF::onActivated(RTC::UniqueId ec_id)
{


   target_x=0.0,target_y=0.0;
   vision_x=0.0,vision_y=0.0;

	m_HumanPosXY.x=target_x;
    m_HumanPosXY.y=target_y;
    m_HumanPosXYOut.write();

	m_HumanPos.x=target_x;
    m_HumanPos.y=target_y;
    m_HumanPos.theta=0;
    m_HumanPosOut.write();

    PortServiceList* portlist;
    portlist = this->get_ports();
    port_connect_check=new bool [portlist->length()];

    for (CORBA::ULong i(0), n(portlist->length()); i < n; ++i)
	{
	    PortService_ptr port;
	    port = (*portlist)[i];
	    cout << "PortNum"<<i<<":"<<port->get_port_profile()->name << "=";
	    if(port->get_port_profile()->connector_profiles.length()!=0){
		port_connect_check[i]=true;
		cout <<"Connect"<<endl;
	    }
	    else if(port->get_port_profile()->connector_profiles.length()==0){
		port_connect_check[i]=false;
		cout <<"DisConnect"<<endl;
	    }
	}
    cout <<"-----------------------"<<endl;


    PortService_ptr port;
    port = (*portlist)[8];//This port is ImageInformation Service Port
    if(port->get_port_profile()->connector_profiles.length()!=0){
		cout<<"LRF Information connect"<<endl;


	lrf_scanpoint=m_LRF->getTotalScanPoints();
	lrf_resolusion=m_LRF->getResolution();
	lrf_offset=m_LRF->getOffsetFOV();


		cout<<"TotalScanPoints:"<<(int)m_LRF->getTotalScanPoints()<<endl;
		cout<<"Resolution:"<<(double)m_LRF->getResolution()<<endl;
	 	cout<<"OffsetFOV:"<<(double)m_LRF->getOffsetFOV()<<endl;
	 	cout<<"Start"<<endl;
		g_distance=new long int[lrf_scanpoint];
		point_x=new double[lrf_scanpoint];
		point_y=new double[lrf_scanpoint];

		obstacle_x=new double[lrf_scanpoint];
		obstacle_y=new double[lrf_scanpoint];
		obstacle_num=0;
		target_px=new double[lrf_scanpoint];
		target_py=new double[lrf_scanpoint];
		target_pnum=0;

    }
    else {
		cout<<"LRF Information Disconnect"<<endl;
		cout<<"Top-URG Parameter Set"<<endl;
		lrf_scanpoint=1081;
		lrf_resolusion=2*PI/1440.0;
//		lrf_scanpoint=769;
//		lrf_resolusion=2*PI/1081.0;
		lrf_offset=(-45)/180.0*PI;
		g_distance=new long int[lrf_scanpoint];
		point_x=new double[lrf_scanpoint];
		point_y=new double[lrf_scanpoint];

		obstacle_x=new double[lrf_scanpoint];
		obstacle_y=new double[lrf_scanpoint];
		obstacle_num=0;
		target_px=new double[lrf_scanpoint];
		target_py=new double[lrf_scanpoint];
		target_pnum=0;

    }
    cout <<"-----------------------"<<endl;


	m_LRFTracking.distance=g_distance;
	m_LRFTracking.point_x=point_x;
	m_LRFTracking.point_y=point_y;
	
	m_LRFTracking.obstacle_x=obstacle_x;
	m_LRFTracking.obstacle_y=obstacle_y;
	m_LRFTracking.target_px=target_px;
	m_LRFTracking.target_py=target_py;
	m_LRFTracking.obstacle_num=obstacle_num;
	m_LRFTracking.target_pnum=target_pnum;

	m_LRFTracking.lrf_scanpoint=lrf_scanpoint;
	m_LRFTracking.lrf_resolusion=lrf_resolusion;
	m_LRFTracking.lrf_offset=lrf_offset;


  return RTC::RTC_OK;
}


RTC::ReturnCode_t HumanDetection_LRF::onDeactivated(RTC::UniqueId ec_id)
{
   target_x=0.0,target_y=0.0;
   vision_x=0.0,vision_y=0.0;

	m_HumanPosXY.x=target_x;
    m_HumanPosXY.y=target_y;
    m_HumanPosXYOut.write();

	m_HumanPos.x=target_x;
    m_HumanPos.y=target_y;
    m_HumanPos.theta=0;
    m_HumanPosOut.write();


	delete[] point_x;
	delete[] point_y;
	delete[] g_distance;

	delete[] obstacle_x;
	delete[] obstacle_y;
	delete[] target_px;
	delete[] target_py;

  return RTC::RTC_OK;
}

//extern double input_x,input_y;
RTC::ReturnCode_t HumanDetection_LRF::onExecute(RTC::UniqueId ec_id)
{

	usleep(10*1000);
    if (m_VisionPosIn.isNew()) {
	m_VisionPosIn.read();
	vision_x=m_VisionPos.x;
	vision_y=m_VisionPos.y;
    }

 
    if (m_UrgDataIn.isNew()) {
		m_UrgDataIn.read();
	//	cout<<m_UrgData.data.length()<<endl;   
		if(m_UrgData.data.length()==lrf_scanpoint){

		    memcpy(g_distance,(void *)&(m_UrgData.data[0]),lrf_scanpoint*sizeof(long int));

			double CYCLE=2*PI/lrf_resolusion;
			double POINTS=lrf_scanpoint;
			double ZeroPos=lrf_offset/(2*PI)*CYCLE-CYCLE/4.0;
	    	double crad;

	    for(int i = 0;i < POINTS;i++){ 
	  	    crad=2*PI*(double)(i+ZeroPos)/CYCLE;
		    point_x[i]=(-1.0)*(double)g_distance[i]*sin(crad)/1000.0;
		    point_y[i]=(double)g_distance[i]*cos(crad)/1000.0;
		   }
			GL.SetLRF(lrf_scanpoint,point_x,point_y);
			}
    }



	//GL.Input(m_LRFTracking.id_flag,m_LRFTracking.goal_urg_x,m_LRFTracking.goal_urg_y);
        int fl=0;

        if (m_TargetOdometryIn.isNew()){
          m_TargetOdometryIn.read();
          target_x=m_TargetOdometry.x;
          target_y=m_TargetOdometry.y;
          GL.SetObjectXY(target_x,target_y);
          fl++;
        }        

        m_LRFTracking.discriminant(vision_x,vision_y,target_x,target_y);

        if (fl==1){
          GL.SetObjectXY(target_x,target_y);
          fl++;
        } 
        
	GL.SetObstacle(m_LRFTracking.obstacle_num,m_LRFTracking.obstacle_x ,m_LRFTracking.obstacle_y );
	GL.SetObject(m_LRFTracking.target_pnum,m_LRFTracking.target_px ,m_LRFTracking.target_py);
	//GL.SetObjectXY(m_LRFTracking.target_x,m_LRFTracking.target_y);

	obstacle_num=m_LRFTracking.obstacle_num;

    m_HumanObject.data.length(target_pnum);
    for(int i=0;i<target_pnum;i++){
	m_HumanObject.data[i].x=target_px[i];
	m_HumanObject.data[i].y=target_py[i];
    }
    m_HumanObjectOut.write();

    m_ObstacleObject.data.length(obstacle_num);
    for(int i=0;i<obstacle_num;i++){
	m_ObstacleObject.data[i].x=obstacle_x[i];
	m_ObstacleObject.data[i].y=obstacle_y[i];
    }
	if(obstacle_num!=0)
    m_ObstacleObjectOut.write();

    m_Obstacle.obstacles.length(obstacle_num);
    for(int i=0;i<obstacle_num;i++){
	m_Obstacle.obstacles[i].x=-obstacle_x[i];
	m_Obstacle.obstacles[i].y=obstacle_y[i];
    m_Obstacle.obstacles[i].r = 0.05;

    }
	if(obstacle_num!=0)
    m_ObstacleOut.write();


	m_HumanPosXY.x=target_x;
    m_HumanPosXY.y=target_y;
    m_HumanPosXYOut.write();

	m_HumanPos.x=target_x;
    m_HumanPos.y=target_y;
    m_HumanPos.theta=0;
    m_HumanPosOut.write();

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t HumanDetection_LRF::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t HumanDetection_LRF::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t HumanDetection_LRF::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t HumanDetection_LRF::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t HumanDetection_LRF::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


extern "C"
{
 
  void HumanDetection_LRFInit(RTC::Manager* manager)
  {
    RTC::Properties profile(humandetection_lrf_spec);
    manager->registerFactory(profile,
                             RTC::Create<HumanDetection_LRF>,
                             RTC::Delete<HumanDetection_LRF>);
  }
  
};



